### Persefone - a socio-economic-ecological model of European agricultural landscapes.
###
### This file contains the functions for calculating insect biomass.
###

##FIXME Updating every pixel every day is really expensive! I should seriously
## think about switching this to a "calculate-on-demand" design.
## (The problem is that that works for all models except the pesticide one, which
## is a Markov Chain model that cannot be calculated without knowing the previous
## value at the location.)

##TODO write tests for this

"""
    updateinsectbiomass!(model)

Calculate a new insect biomass value for each pixel in the landscape.
"""
function updateinsectbiomass!(model::AgentBasedModel)
    @debug "Updating insect biomass"
    if @param(nature.insectmodel) == "seasonal"
        updatefunction = seasonalmodel
    elseif @param(nature.insectmodel) == "seasonalhabitat"
        updatefunction = seasonalhabitatmodel
    else
        Base.error("Unknown insect model $(@param(nature.insectmodel)).")
        #TODO replace with exception
    end
    for p in model.landscape
        p.insectbiomass = updatefunction(p, model)
    end
end


"""
    seasonalmodel(pixel)

Calculate an insect biomass value based purely on the calendar day, following
a parabolic function as per Paquette et al. (2013). Assumes a maximum insect
density of 300 mg/m² (cf. Odderskaer et al. 1997, Püttmanns et al. 2022).

*[Note that this is a very approximate calculation!]*
"""
function seasonalmodel(pixel::Pixel, model::AgentBasedModel)
    calendarday = dayofyear(model.date)
    biomass = -0.085(calendarday-187)^2+300
    biomass > 0 ? biomass : 0
end

"""
    seasonalhabitatmodel(pixel)

Calculate an insect biomass value based on the calendar day and habitat type,
following a parabolic function.

See Odderskaer et al. (1997), Grübler et al. (2008), Paquette et al. (2013),
and Püttmanns et al. (2022).

*[Note that this is a very approximate calculation!]*
"""
function seasonalhabitatmodel(pixel::Pixel, model::AgentBasedModel)
    calendarday = dayofyear(model.date)
    habitatfactor = 0.0
    if pixel.landcover == soil || pixel.landcover == forest
        habitatfactor = 2.0
    elseif pixel.landcover == grass
        habitatfactor = 1.5
    elseif pixel.landcover == agriculture
        habitatfactor = 1.0
    end
    biomass = -0.085(calendarday-187)^2+300*habitatfactor
    biomass > 0 ? biomass : 0
end

#TODO add models that include weather and pesticide application
